from typing import Any

import yaml
import os
import codecs
from .env_loader import EnvLoader


class Config:
    def __init__(self, *config_dirs: str) -> None:
        self.config = dict[str, Any]()

        for config_dir in config_dirs:
            self.config.update(self.load_config(config_dir))

        if not self.config:
            raise ValueError(f"Config is empty! Check {config_dirs=}")

    def get(self) -> dict[str, Any]:
        return self.config

    @staticmethod
    def load_config(path: str | None = None) -> dict[str, Any]:
        if not path:
            return {}
        if path.endswith("/"):
            path = path[:-1]
        if not os.path.exists(path):
            raise FileNotFoundError(f"Config is not present! {path=}")
        with codecs.open(f"{path}/config.yaml", encoding="utf-8") as file:
            config: dict[str, Any] = yaml.load(file, EnvLoader)
            return config



from typing import Any

from yaml import Node, SafeLoader
import re
import os

ENV_TAG = "!ENV"
ENV_PATTERN: re.Pattern = re.compile(r"\$([^{}]+|\{[^{}]+})")  # accept either $VAR or ${VAR}


class EnvLoader(SafeLoader):
    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)

        # consider each ENV_PATTERN match to be tagged with ENV_TAG
        self.add_implicit_resolver(ENV_TAG, ENV_PATTERN, None)  # type: ignore[no-untyped-call]

        # construct each node with ENV_TAG according to env_constructor
        self.add_constructor(ENV_TAG, self.env_constructor)

    # clumsy definition but I wanted to avoid calling `add_constructor` outside of __init__
    @staticmethod
    def env_constructor(_: "EnvLoader", node: Node) -> Any:
        return os.path.expandvars(node.value)
